@@ -8,5 +8,4 @@
 
 #undef NEED_SNPRINTF_H
 
-/* Define this to use the wiretap library */
-#undef WITH_WIRETAP
+#undef NEED_STRERROR_H
