@@ -1,7 +1,7 @@
 /* etypes.h
  * Defines ethernet packet types, similar to tcpdump's ethertype.h
  *
- * $Id: etypes.h,v 1.4 1998/12/19 00:12:20 hannes Exp $
+ * $Id: etypes.h,v 1.8 1999/08/22 01:35:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -26,6 +26,13 @@
 #ifndef __ETYPES_H__
 #define __ETYPES_H__
 
+/*
+ * Maximum length of an IEEE 802.3 frame; Ethernet type/length values
+ * greater than it are types, Ethernet type/length values less than or
+ * equal to it are lengths.
+ */
+#define IEEE_802_3_MAX_LEN 1500
+
 #ifndef ETHERTYPE_UNK
 #define ETHERTYP_UNK 0x0000
 #endif
@@ -70,6 +77,11 @@
 #define ETHERTYPE_VINES 0xbad
 #endif
 
+#ifndef ETHERTYPE_TRAIN
+#define ETHERTYPE_TRAIN 0x1984
+/* Created by Netmon as a summary packet */
+#endif
+
 #ifndef ETHERTYPE_CDP
 #define ETHERTYPE_CDP 0x2000	/* Cisco Discovery Protocol */
 #endif
@@ -78,4 +90,15 @@
 #define ETHERTYPE_LOOP 0x9000 	/* used for layer 2 testing (do i see my own frames on the wire) */
 #endif
 
+#ifndef ETHERTYPE_PPPOED
+#define ETHERTYPE_PPPOED 0x8863 /* PPPoE Discovery Protocol */
+#endif
+
+#ifndef ETHERTYPE_PPPOES
+#define ETHERTYPE_PPPOES 0x8864 /* PPPoE Session Protocol */
+#endif
+
+
 #endif /* etypes.h */
+
+
