@@ -1,7 +1,7 @@
 /* packet-arp.c
  * Routines for ARP packet disassembly
  *
- * $Id: packet-arp.c,v 1.12 1999/03/23 03:14:35 gram Exp $
+ * $Id: packet-arp.c,v 1.17 1999/09/12 06:11:35 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,6 +35,8 @@
 #include "packet.h"
 #include "etypes.h"
 
+static int proto_arp = -1;
+
 /* Definitions taken from Linux "linux/if_arp.h" header file, and from
 
 	http://www.isi.edu/in-notes/iana/assignments/arp-parameters
@@ -71,9 +73,6 @@
 #define ARPHRD_TWINAX	26		/* Twinaxial			*/
 #define ARPHRD_EUI_64	27		/* EUI-64			*/
 
-/* Max string length for displaying unknown type of ARP address.  */
-#define	MAX_ADDR_STR_LEN	16
-
 /* ARP / RARP structs and definitions */
 #ifndef ARPOP_REQUEST
 #define ARPOP_REQUEST  1       /* ARP request.  */
@@ -89,41 +88,6 @@
 #define ARPOP_RREPLY   4       /* RARP reply.  */
 #endif
 
-static gchar *
-arpaddr_to_str(guint8 *ad, int ad_len) {
-  static gchar  str[3][MAX_ADDR_STR_LEN+3+1];
-  static gchar *cur;
-  gchar        *p;
-  int           len;
-  static const char hex[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
-                                '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
-
-  if (cur == &str[0][0]) {
-    cur = &str[1][0];
-  } else if (cur == &str[1][0]) {  
-    cur = &str[2][0];
-  } else {  
-    cur = &str[0][0];
-  }
-  p = cur;
-  len = MAX_ADDR_STR_LEN;
-  while (ad_len > 0 && len > 0) {
-    *p++ = hex[(*ad) >> 4];
-    *p++ = hex[(*ad) & 0xF];
-    len -= 2;
-    ad++;
-    ad_len--;
-  }
-  if (ad_len != 0) {
-    /* Note that we're not showing the full address.  */
-    *p++ = '.';
-    *p++ = '.';
-    *p++ = '.';
-  }
-  *p = '\0';
-  return cur;
-}
-
 gchar *
 arphrdaddr_to_str(guint8 *ad, int ad_len, guint16 type) {
   if ((type == ARPHRD_ETHER || type == ARPHRD_EETHER || type == ARPHRD_IEEE802)
@@ -132,7 +96,7 @@ arphrdaddr_to_str(guint8 *ad, int ad_len, guint16 type) {
        address, which are the same type of address). */
     return ether_to_str(ad);
   }
-  return arpaddr_to_str(ad, ad_len);
+  return bytes_to_str(ad, ad_len);
 }
 
 static gchar *
@@ -141,7 +105,7 @@ arpproaddr_to_str(guint8 *ad, int ad_len, guint16 type) {
     /* IP address.  */
     return ip_to_str(ad);
   }
-  return arpaddr_to_str(ad, ad_len);
+  return bytes_to_str(ad, ad_len);
 }
 
 gchar *
@@ -224,10 +188,19 @@ dissect_arp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   tpa_str = arpproaddr_to_str((guint8 *) &pd[tpa_offset], ar_pln, ar_pro);
   
   if (check_col(fd, COL_PROTOCOL)) {
-    if ((op_str = match_strval(ar_op, op_vals)))
-      col_add_str(fd, COL_PROTOCOL, op_str);
-    else
+    switch (ar_op) {
+
+    case ARPOP_REQUEST:
+    case ARPOP_REPLY:
+    default:
       col_add_str(fd, COL_PROTOCOL, "ARP");
+      break;
+
+    case ARPOP_RREQUEST:
+    case ARPOP_RREPLY:
+      col_add_str(fd, COL_PROTOCOL, "RARP");
+      break;
+    }
   }
 
   if (check_col(fd, COL_INFO)) {
@@ -254,30 +227,41 @@ dissect_arp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
   if (tree) {
     if ((op_str = match_strval(ar_op, op_vals)))
-      ti = proto_tree_add_item(tree, offset, 8 + 2*ar_hln + 2*ar_pln,
-        op_str);
+      ti = proto_tree_add_item_format(tree, proto_arp, offset,
+        8 + 2*ar_hln + 2*ar_pln, NULL, op_str);
     else
-      ti = proto_tree_add_item(tree, offset, 8 + 2*ar_hln + 2*ar_pln,
-        "Unknown ARP (opcode 0x%04x)", ar_op);
-    arp_tree = proto_tree_new();
-    proto_item_add_subtree(ti, arp_tree, ETT_ARP);
-    proto_tree_add_item(arp_tree, offset + AR_HRD, 2,
+      ti = proto_tree_add_item_format(tree, proto_arp, offset,
+        8 + 2*ar_hln + 2*ar_pln, NULL, "Unknown ARP (opcode 0x%04x)", ar_op);
+    arp_tree = proto_item_add_subtree(ti, ETT_ARP);
+    proto_tree_add_text(arp_tree, offset + AR_HRD, 2,
       "Hardware type: %s", arphrdtype_to_str(ar_hrd, "Unknown (0x%04x)"));
-    proto_tree_add_item(arp_tree, offset + AR_PRO, 2,
-      "Protocol type: %s", ethertype_to_str(ar_pro, "Unknown (0x%04x)"));
-    proto_tree_add_item(arp_tree, offset + AR_HLN, 1,
+    proto_tree_add_text(arp_tree, offset + AR_PRO, 2,
+      "Protocol type: %s", val_to_str(ar_pro, etype_vals, "Unknown (0x%04x)"));
+    proto_tree_add_text(arp_tree, offset + AR_HLN, 1,
       "Hardware size: %d", ar_hln);
-    proto_tree_add_item(arp_tree, offset + AR_PLN, 1,
+    proto_tree_add_text(arp_tree, offset + AR_PLN, 1,
       "Protocol size: %d", ar_pln);
-    proto_tree_add_item(arp_tree, offset + AR_OP,  2,
+    proto_tree_add_text(arp_tree, offset + AR_OP,  2,
       "Opcode: 0x%04x (%s)", ar_op, op_str ? op_str : "Unknown");
-    proto_tree_add_item(arp_tree, sha_offset, ar_hln,
+    proto_tree_add_text(arp_tree, sha_offset, ar_hln,
       "Sender hardware address: %s", sha_str);
-    proto_tree_add_item(arp_tree, spa_offset, ar_pln,
+    proto_tree_add_text(arp_tree, spa_offset, ar_pln,
       "Sender protocol address: %s", spa_str);
-    proto_tree_add_item(arp_tree, tha_offset, ar_hln,
+    proto_tree_add_text(arp_tree, tha_offset, ar_hln,
       "Target hardware address: %s", tha_str);
-    proto_tree_add_item(arp_tree, tpa_offset, ar_pln,
+    proto_tree_add_text(arp_tree, tpa_offset, ar_pln,
       "Target protocol address: %s", tpa_str);
   }
 }
+
+void
+proto_register_arp(void)
+{
+/*        static hf_register_info hf[] = {
+                { &variable,
+                { "Name",           "arp.abbreviation", TYPE, VALS_POINTER }},
+        };*/
+
+        proto_arp = proto_register_protocol("Address Resolution Protocol", "arp");
+ /*       proto_register_field_array(proto_arp, hf, array_length(hf));*/
+}
