@@ -1,7 +1,7 @@
 /* packet-dns.c
  * Routines for DNS packet disassembly
  *
- * $Id: packet-dns.c,v 1.17 1999/03/23 03:14:36 gram Exp $
+ * $Id: packet-dns.c,v 1.21 1999/09/21 07:15:38 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -40,6 +40,7 @@
 #include "packet-dns.h"
 #include "util.h"
 
+static int proto_dns = -1;
 
 /* DNS structs and definitions */
 
@@ -287,100 +288,77 @@ dns_class_name(int class)
 
   return class_name;
 }
-  
-
-static int
-is_compressed_name(const u_char *foo)
-{
-  return (0xc0 == (*foo & 0xc0));
-}
-
-
-static int
-get_compressed_name_offset(const u_char *ptr)
-{
-  return ((*ptr & ~0xc0) << 8) | *(ptr+1);
-}
-
-
-static int
-copy_one_name_component(const u_char *dataptr, char *nameptr)
-{
-  int len;
-  int n;
-  
-  len = n  = *dataptr++;
-  if (0 == len)
-    return 0;
-  
-  while (n-- > 0)
-    *nameptr++ = *dataptr++;
-
-  return len;
-}
-
-
-static int
-copy_name_component_rec(const u_char *dns_data_ptr, const u_char *dataptr,
-  char *nameptr, int *real_string_len)
-{
-  int len = 0;
-  int str_len;
-  int offset;
-  int compress = 0;
-  
-  if (is_compressed_name(dataptr)) {
-    compress = 1;
-    offset = get_compressed_name_offset(dataptr);
-    dataptr = dns_data_ptr + offset;
-    copy_name_component_rec(dns_data_ptr, dataptr, nameptr, &str_len);
-    *real_string_len += str_len;
-    nameptr += str_len;
-    len = 2;
-  }
-  else {
-    str_len = copy_one_name_component(dataptr, nameptr);
-    *real_string_len = str_len;
-    dataptr += str_len + 1;
-    len     += str_len + 1;
-    nameptr += str_len;
-  }
-
-  if (compress)
-    return len;
-  
-  (*real_string_len)++;
-
-  if (*dataptr > 0) {
-    *nameptr++ = '.';
-    len += copy_name_component_rec(dns_data_ptr, dataptr, nameptr, &str_len);
-    *real_string_len += str_len;
-    return len;
-  }
-
-  return len + 1;
-}
-
 
 int
-get_dns_name(const u_char *dns_data_ptr, const u_char *pd, int offset,
-  char *nameptr, int maxname)
+get_dns_name(const u_char *dns_data_ptr, const u_char *dptr, char *name,
+    int maxname)
 {
-  int len;
-  const u_char *dataptr = pd + offset;
-  int str_len = 0;
+  const u_char *dp = dptr;
+  char *np = name;
+  int len = -1;
+  u_int component_len;
+  int offset;
 
-  memset (nameptr, 0, maxname);
-  len = copy_name_component_rec(dns_data_ptr, dataptr, nameptr, &str_len);
-  
+  maxname--;	/* reserve space for the trailing '\0' */
+  while ((component_len = *dp++) != 0) {
+    switch (component_len & 0xc0) {
+
+    case 0x00:
+      /* Label */
+      if (np != name) {
+      	/* Not the first component - put in a '.'. */
+        if (maxname > 0) {
+          *np++ = '.';
+          maxname--;
+        }
+      }
+      while (component_len > 0) {
+        if (maxname > 0) {
+          *np++ = *dp;
+          maxname--;
+        }
+      	component_len--;
+      	dp++;
+      }
+      break;
+
+    case 0x40:
+    case 0x80:
+      goto error;	/* error */
+
+    case 0xc0:
+      /* Pointer. */
+      /* XXX - check to make sure we aren't looping, by keeping track
+         of how many characters are in the DNS packet, and of how many
+         characters we've looked at, and quitting if the latter
+         becomes bigger than the former. */
+      offset = ((component_len & ~0xc0) << 8) | *dp++;
+      /* If "len" is negative, we are still working on the original name,
+         not something pointed to by a pointer, and so we should set "len"
+         to the length of the original name. */
+      if (len < 0)
+        len = dp - dptr;
+      dp = dns_data_ptr + offset;
+      break;	/* now continue processing from there */
+    }
+  }
+        
+error:
+  *np = '\0';
+  /* If "len" is negative, we haven't seen a pointer, and thus haven't
+     set the length, so set it. */
+  if (len < 0)
+    len = dp - dptr;
+  /* Zero-length name means "root server" */
+  if (*name == '\0')
+    strcpy(name, "<Root>");
   return len;
 }
 
 
 static int
 get_dns_name_type_class (const u_char *dns_data_ptr,
-			 const u_char *pd,
-			 int offset,
+			 const u_char *dptr,
 			 char *name_ret,
 			 int *name_len_ret,
 			 int *type_ret,
@@ -391,27 +369,68 @@ get_dns_name_type_class (const u_char *dns_data_ptr,
   int type;
   int class;
   char name[MAXDNAME];
-  const u_char *pd_save;
+  const u_char *dptr_save;
 
-  name_len = get_dns_name(dns_data_ptr, pd, offset, name, sizeof(name));
-  pd += offset;
-  pd_save = pd;
-  pd += name_len;
+  name_len = get_dns_name(dns_data_ptr, dptr, name, sizeof(name));
+  dptr_save = dptr;
+  dptr += name_len;
   
-  type = pntohs(pd);
-  pd += 2;
-  class = pntohs(pd);
-  pd += 2;
+  type = pntohs(dptr);
+  dptr += 2;
+  class = pntohs(dptr);
+  dptr += 2;
 
   strcpy (name_ret, name);
   *type_ret = type;
   *class_ret = class;
   *name_len_ret = name_len;
 
-  len = pd - pd_save;
+  len = dptr - dptr_save;
   return len;
 }
 
+static double
+rfc1867_size(const u_char *dptr)
+{
+  double size;
+  guint32 exponent;
+
+  size = (*dptr & 0xF0) >> 4;
+  exponent = (*dptr & 0x0F);
+  while (exponent != 0) {
+    size *= 10;
+    exponent--;
+  }
+  return size / 100;	/* return size in meters, not cm */
+}
+
+static char *
+rfc1867_angle(const u_char *dptr, const char *nsew)
+{
+  guint32 angle;
+  char direction;
+  guint32 degrees, minutes, secs, tsecs;
+  static char buf[10+1+3+1 + 2+1+3+1 + 2+1+3+1+3+1 + 1 + 1];
+
+  angle = pntohl(dptr);
+
+  if (angle < 0x80000000U) {
+    angle = 0x80000000U - angle;
+    direction = nsew[1];
+  } else {
+    angle = angle - 0x80000000U;
+    direction = nsew[0];
+  }
+  tsecs = angle % 1000;
+  angle = angle / 1000;
+  secs = angle % 60;
+  angle = angle / 60;
+  minutes = angle % 60;
+  degrees = angle / 60;
+  sprintf(buf, "%u deg %u min %u.%03u sec %c", degrees, minutes, secs,
+		tsecs, direction);
+  return buf;
+}
 
 static int
 dissect_dns_query(const u_char *dns_data_ptr, const u_char *pd, int offset,
@@ -432,7 +451,7 @@ dissect_dns_query(const u_char *dns_data_ptr, const u_char *pd, int offset,
 
   data_start = dptr = pd + offset;
 
-  len = get_dns_name_type_class(dns_data_ptr, pd, offset, name, &name_len,
+  len = get_dns_name_type_class(dns_data_ptr, dptr, name, &name_len,
     &type, &class);
   dptr += len;
 
@@ -440,18 +459,17 @@ dissect_dns_query(const u_char *dns_data_ptr, const u_char *pd, int offset,
   class_name = dns_class_name(class);
   long_type_name = dns_long_type_name(type);
 
-  tq = proto_tree_add_item(dns_tree, offset, len, "%s: type %s, class %s", 
+  tq = proto_tree_add_text(dns_tree, offset, len, "%s: type %s, class %s", 
 		   name, type_name, class_name);
-  q_tree = proto_tree_new();
-  proto_item_add_subtree(tq, q_tree, ETT_DNS_QD);
+  q_tree = proto_item_add_subtree(tq, ETT_DNS_QD);
 
-  proto_tree_add_item(q_tree, offset, name_len, "Name: %s", name);
+  proto_tree_add_text(q_tree, offset, name_len, "Name: %s", name);
   offset += name_len;
 
-  proto_tree_add_item(q_tree, offset, 2, "Type: %s", long_type_name);
+  proto_tree_add_text(q_tree, offset, 2, "Type: %s", long_type_name);
   offset += 2;
 
-  proto_tree_add_item(q_tree, offset, 2, "Class: %s", class_name);
+  proto_tree_add_text(q_tree, offset, 2, "Class: %s", class_name);
   offset += 2;
   
   return dptr - data_start;
@@ -465,18 +483,17 @@ add_rr_to_tree(proto_item *trr, int rr_type, int offset, const char *name,
 {
   proto_tree *rr_tree;
 
-  rr_tree = proto_tree_new();
-  proto_item_add_subtree(trr, rr_tree, rr_type);
-  proto_tree_add_item(rr_tree, offset, namelen, "Name: %s", name);
+  rr_tree = proto_item_add_subtree(trr, rr_type);
+  proto_tree_add_text(rr_tree, offset, namelen, "Name: %s", name);
   offset += namelen;
-  proto_tree_add_item(rr_tree, offset, 2, "Type: %s", type_name);
+  proto_tree_add_text(rr_tree, offset, 2, "Type: %s", type_name);
   offset += 2;
-  proto_tree_add_item(rr_tree, offset, 2, "Class: %s", class_name);
+  proto_tree_add_text(rr_tree, offset, 2, "Class: %s", class_name);
   offset += 2;
-  proto_tree_add_item(rr_tree, offset, 4, "Time to live: %s",
+  proto_tree_add_text(rr_tree, offset, 4, "Time to live: %s",
 						time_secs_to_str(ttl));
   offset += 4;
-  proto_tree_add_item(rr_tree, offset, 2, "Data length: %u", data_len);
+  proto_tree_add_text(rr_tree, offset, 2, "Data length: %u", data_len);
   return rr_tree;
 }
 
@@ -498,10 +515,11 @@ dissect_dns_answer(const u_char *dns_data_ptr, const u_char *pd, int offset,
   u_short data_len;
   proto_tree *rr_tree;
   proto_item *trr;
+  const u_char *rrptr;
 
   data_start = dptr = pd + offset;
 
-  len = get_dns_name_type_class(dns_data_ptr, pd, offset, name, &name_len,
+  len = get_dns_name_type_class(dns_data_ptr, dptr, name, &name_len,
     &type, &class);
   dptr += len;
 
@@ -516,63 +534,182 @@ dissect_dns_answer(const u_char *dns_data_ptr, const u_char *pd, int offset,
   dptr += 2;
 
   switch (type) {
-  case T_A: 		/* "A" record */
-    trr = proto_tree_add_item(dns_tree, offset, (dptr - data_start) + data_len,
+  case T_A:
+    trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
 		     "%s: type %s, class %s, addr %s",
 		     name, type_name, class_name,
 		     ip_to_str((guint8 *)dptr));
     rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
                      long_type_name, class_name, ttl, data_len);
     offset += (dptr - data_start);
-    proto_tree_add_item(rr_tree, offset, 4, "Addr: %s",
+    proto_tree_add_text(rr_tree, offset, 4, "Addr: %s",
                      ip_to_str((guint8 *)dptr));
     break;
 
-  case T_NS: 		/* "NS" record */
+  case T_NS:
     {
       char ns_name[MAXDNAME];
       int ns_name_len;
       
-      ns_name_len = get_dns_name(dns_data_ptr, dptr, 0, ns_name, sizeof(ns_name));
-      trr = proto_tree_add_item(dns_tree, offset, (dptr - data_start) + data_len,
+      ns_name_len = get_dns_name(dns_data_ptr, dptr, ns_name, sizeof(ns_name));
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
 		       "%s: type %s, class %s, ns %s",
 		       name, type_name, class_name, ns_name);
       rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
                        long_type_name, class_name, ttl, data_len);
       offset += (dptr - data_start);
-      proto_tree_add_item(rr_tree, offset, ns_name_len, "Name server: %s", ns_name);
+      proto_tree_add_text(rr_tree, offset, ns_name_len, "Name server: %s", ns_name);
     }
     break;
 
-  case T_CNAME:		/* "CNAME" record */
+  case T_CNAME:
     {
       char cname[MAXDNAME];
       int cname_len;
       
-      cname_len = get_dns_name(dns_data_ptr, dptr, 0, cname, sizeof(cname));
-      trr = proto_tree_add_item(dns_tree, offset, (dptr - data_start) + data_len,
+      cname_len = get_dns_name(dns_data_ptr, dptr, cname, sizeof(cname));
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
 		     "%s: type %s, class %s, cname %s",
 		     name, type_name, class_name, cname);
       rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
                        long_type_name, class_name, ttl, data_len);
       offset += (dptr - data_start);
-      proto_tree_add_item(rr_tree, offset, data_len, "Primary name: %s", cname);
+      proto_tree_add_text(rr_tree, offset, cname_len, "Primary name: %s", cname);
     }
     break;
 
-  case T_PTR:		/* "PTR" record */
+  case T_SOA:
+    {
+      char mname[MAXDNAME];
+      int mname_len;
+      char rname[MAXDNAME];
+      int rname_len;
+      guint32 serial;
+      guint32 refresh;
+      guint32 retry;
+      guint32 expire;
+      guint32 minimum;
+
+      rrptr = dptr;
+      mname_len = get_dns_name(dns_data_ptr, rrptr, mname, sizeof(mname));
+      rrptr += mname_len;
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
+		     "%s: type %s, class %s, mname %s",
+		     name, type_name, class_name, mname);
+      rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
+                       long_type_name, class_name, ttl, data_len);
+      offset += (dptr - data_start);
+      proto_tree_add_text(rr_tree, offset, mname_len, "Primary name server: %s",
+                       mname);
+      offset += mname_len;
+      rname_len = get_dns_name(dns_data_ptr, rrptr, rname, sizeof(rname));
+      proto_tree_add_text(rr_tree, offset, rname_len, "Responsible authority's mailbox: %s",
+                       rname);
+      rrptr += rname_len;
+      offset += rname_len;
+      serial = pntohl(rrptr);
+      proto_tree_add_text(rr_tree, offset, 4, "Serial number: %u",
+                       serial);
+      rrptr += 4;
+      offset += 4;
+      refresh = pntohl(rrptr);
+      proto_tree_add_text(rr_tree, offset, 4, "Refresh interval: %s",
+                       time_secs_to_str(refresh));
+      rrptr += 4;
+      offset += 4;
+      retry = pntohl(rrptr);
+      proto_tree_add_text(rr_tree, offset, 4, "Retry interval: %s",
+                       time_secs_to_str(retry));
+      rrptr += 4;
+      offset += 4;
+      expire = pntohl(rrptr);
+      proto_tree_add_text(rr_tree, offset, 4, "Expiration limit: %s",
+                       time_secs_to_str(expire));
+      rrptr += 4;
+      offset += 4;
+      minimum = pntohl(rrptr);
+      proto_tree_add_text(rr_tree, offset, 4, "Minimum TTL: %s",
+                       time_secs_to_str(minimum));
+    }
+    break;
+
+  case T_PTR:
     {
       char pname[MAXDNAME];
       int pname_len;
       
-      pname_len = get_dns_name(dns_data_ptr, dptr, 0, pname, sizeof(pname));
-      trr = proto_tree_add_item(dns_tree, offset, (dptr - data_start) + data_len,
+      pname_len = get_dns_name(dns_data_ptr, dptr, pname, sizeof(pname));
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
 		     "%s: type %s, class %s, ptr %s",
 		     name, type_name, class_name, pname);
       rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
                        long_type_name, class_name, ttl, data_len);
       offset += (dptr - data_start);
-      proto_tree_add_item(rr_tree, offset, data_len, "Domain name: %s", pname);
+      proto_tree_add_text(rr_tree, offset, pname_len, "Domain name: %s", pname);
+      break;
+    }
+    break;
+
+  case T_MX:
+    {
+      guint16 preference;
+      char mx_name[MAXDNAME];
+      int mx_name_len;
+      
+      rrptr = dptr;
+      preference = pntohs(rrptr);
+      rrptr += 2;
+      mx_name_len = get_dns_name(dns_data_ptr, rrptr, mx_name, sizeof(mx_name));
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
+		       "%s: type %s, class %s, preference %u, mx %s",
+		       name, type_name, class_name, preference, mx_name);
+      rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
+                       long_type_name, class_name, ttl, data_len);
+      offset += (dptr - data_start);
+      proto_tree_add_text(rr_tree, offset, 2, "Preference: %u", preference);
+      offset += 2;
+      proto_tree_add_text(rr_tree, offset, mx_name_len, "Mail exchange: %s", mx_name);
+    }
+    break;
+      
+  case T_LOC:
+    {
+      rrptr = dptr;
+      trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
+		     "%s: type %s, class %s",
+		     name, type_name, class_name);
+      rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
+                       long_type_name, class_name, ttl, data_len);
+      offset += (dptr - data_start);
+      proto_tree_add_text(rr_tree, offset, 1, "Version: %u", *dptr);
+      if (*rrptr == 0) {
+	/* Version 0, the only version RFC 1876 discusses. */
+	rrptr++;
+	offset++;
+	proto_tree_add_text(rr_tree, offset, 1, "Size: %g m",
+				rfc1867_size(rrptr));
+	rrptr++;
+	offset++;
+	proto_tree_add_text(rr_tree, offset, 1, "Horizontal precision: %g m",
+				rfc1867_size(rrptr));
+	rrptr++;
+	offset++;
+	proto_tree_add_text(rr_tree, offset, 1, "Vertical precision: %g m",
+				rfc1867_size(rrptr));
+	rrptr++;
+	offset++;
+	proto_tree_add_text(rr_tree, offset, 4, "Latitude: %s",
+				rfc1867_angle(rrptr, "NS"));
+	rrptr += 4;
+	offset += 4;
+	proto_tree_add_text(rr_tree, offset, 4, "Longitude: %s",
+				rfc1867_angle(rrptr, "EW"));
+	rrptr += 4;
+	offset += 4;
+	proto_tree_add_text(rr_tree, offset, 4, "Altitude: %g m",
+				(pntohl(rrptr) - 10000000)/100.0);
+      } else
+        proto_tree_add_text(rr_tree, offset, data_len, "Data");
       break;
     }
     break;
@@ -580,13 +717,13 @@ dissect_dns_answer(const u_char *dns_data_ptr, const u_char *pd, int offset,
     /* TODO: parse more record types */
 
   default:
-    trr = proto_tree_add_item(dns_tree, offset, (dptr - data_start) + data_len,
+    trr = proto_tree_add_text(dns_tree, offset, (dptr - data_start) + data_len,
                      "%s: type %s, class %s",
 		     name, type_name, class_name);
     rr_tree = add_rr_to_tree(trr, ETT_DNS_RR, offset, name, name_len,
                        long_type_name, class_name, ttl, data_len);
     offset += (dptr - data_start);
-    proto_tree_add_item(rr_tree, offset, data_len, "Data");
+    proto_tree_add_text(rr_tree, offset, data_len, "Data");
   }
   
   dptr += data_len;
@@ -603,9 +740,8 @@ dissect_query_records(const u_char *dns_data_ptr, int count, const u_char *pd,
   proto_item *ti;
   
   start_off = cur_off;
-  ti = proto_tree_add_item(dns_tree, start_off, 0, "Queries");
-  qatree = proto_tree_new();
-  proto_item_add_subtree(ti, qatree, ETT_DNS_QRY);
+  ti = proto_tree_add_text(dns_tree, start_off, 0, "Queries");
+  qatree = proto_item_add_subtree(ti, ETT_DNS_QRY);
   while (count-- > 0)
     cur_off += dissect_dns_query(dns_data_ptr, pd, cur_off, qatree);
   proto_item_set_len(ti, cur_off - start_off);
@@ -625,9 +761,8 @@ dissect_answer_records(const u_char *dns_data_ptr, int count,
   proto_item *ti;
   
   start_off = cur_off;
-  ti = proto_tree_add_item(dns_tree, start_off, 0, name);
-  qatree = proto_tree_new();
-  proto_item_add_subtree(ti, qatree, ETT_DNS_ANS);
+  ti = proto_tree_add_text(dns_tree, start_off, 0, name);
+  qatree = proto_item_add_subtree(ti, ETT_DNS_ANS);
   while (count-- > 0)
     cur_off += dissect_dns_answer(dns_data_ptr, pd, cur_off, qatree);
   proto_item_set_len(ti, cur_off - start_off);
@@ -678,13 +813,12 @@ dissect_dns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   }
   
   if (tree) {
-    ti = proto_tree_add_item(tree, offset, 4,
+    ti = proto_tree_add_item_format(tree, proto_dns, offset, 4, NULL,
 			  (flags & F_RESPONSE) ? "DNS response" : "DNS query");
     
-    dns_tree = proto_tree_new();
-    proto_item_add_subtree(ti, dns_tree, ETT_DNS);
+    dns_tree = proto_item_add_subtree(ti, ETT_DNS);
     
-    proto_tree_add_item(dns_tree, offset + DNS_ID, 2, "Transaction ID: 0x%04x",
+    proto_tree_add_text(dns_tree, offset + DNS_ID, 2, "Transaction ID: 0x%04x",
     			id);
 
     strcpy(buf, val_to_str(flags & F_OPCODE, opcode_vals, "Unknown operation"));
@@ -694,47 +828,46 @@ dissect_dns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       strcat(buf, val_to_str(flags & F_RCODE, rcode_vals,
             "Unknown error"));
     }
-    tf = proto_tree_add_item(dns_tree, offset + DNS_FLAGS, 2, "Flags: 0x%04x (%s)",
+    tf = proto_tree_add_text(dns_tree, offset + DNS_FLAGS, 2, "Flags: 0x%04x (%s)",
                           flags, buf);
-    field_tree = proto_tree_new();
-    proto_item_add_subtree(tf, field_tree, ETT_DNS_FLAGS);
-    proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+    field_tree = proto_item_add_subtree(tf, ETT_DNS_FLAGS);
+    proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
        decode_boolean_bitfield(flags, F_RESPONSE,
             2*8, "Response", "Query"));
-    proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+    proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
        decode_enumerated_bitfield(flags, F_OPCODE,
             2*8, opcode_vals, "%s"));
     if (flags & F_RESPONSE) {
-      proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+      proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
          decode_boolean_bitfield(flags, F_AUTHORITATIVE,
               2*8,
               "Server is an authority for domain",
               "Server isn't an authority for domain"));
     }
-    proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+    proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
        decode_boolean_bitfield(flags, F_TRUNCATED,
             2*8,
             "Message is truncated",
             "Message is not truncated"));
-    proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+    proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
        decode_boolean_bitfield(flags, F_RECDESIRED,
             2*8,
             "Do query recursively",
             "Don't do query recursively"));
     if (flags & F_RESPONSE) {
-      proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+      proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
          decode_boolean_bitfield(flags, F_RECAVAIL,
               2*8,
               "Server can do recursive queries",
               "Server can't do recursive queries"));
-      proto_tree_add_item(field_tree, offset + DNS_FLAGS, 2, "%s",
+      proto_tree_add_text(field_tree, offset + DNS_FLAGS, 2, "%s",
          decode_enumerated_bitfield(flags, F_RCODE,
               2*8, rcode_vals, "%s"));
     }
-    proto_tree_add_item(dns_tree, offset + DNS_QUEST, 2, "Questions: %d", quest);
-    proto_tree_add_item(dns_tree, offset + DNS_ANS, 2, "Answer RRs: %d", ans);
-    proto_tree_add_item(dns_tree, offset + DNS_AUTH, 2, "Authority RRs: %d", auth);
-    proto_tree_add_item(dns_tree, offset + DNS_ADD, 2, "Additional RRs: %d", add);
+    proto_tree_add_text(dns_tree, offset + DNS_QUEST, 2, "Questions: %d", quest);
+    proto_tree_add_text(dns_tree, offset + DNS_ANS, 2, "Answer RRs: %d", ans);
+    proto_tree_add_text(dns_tree, offset + DNS_AUTH, 2, "Authority RRs: %d", auth);
+    proto_tree_add_text(dns_tree, offset + DNS_ADD, 2, "Additional RRs: %d", add);
 
     cur_off = offset + DNS_HDRLEN;
     
@@ -755,3 +888,15 @@ dissect_dns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
           dns_tree, "Additional records");
   }
 }
+
+void
+proto_register_dns(void)
+{
+/*        static hf_register_info hf[] = {
+                { &variable,
+                { "Name",           "dns.abbreviation", TYPE, VALS_POINTER }},
+        };*/
+
+        proto_dns = proto_register_protocol("Domain Name Service", "dns");
+ /*       proto_register_field_array(proto_dns, hf, array_length(hf));*/
+}
