@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c,v 1.9 1999/03/23 03:14:37 gram Exp $
+ * $Id: packet-eth.c,v 1.19 1999/09/15 06:26:42 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -36,7 +36,16 @@
 #include "etypes.h"
 #include "resolv.h"
 
-#define IEEE_802_3_MAX_LEN 1500
+extern const value_string etype_vals[];
+
+/* protocols and header fields */
+static int proto_eth = -1;
+static int hf_eth_dst = -1;
+static int hf_eth_src = -1;
+static int hf_eth_len = -1;
+static int hf_eth_type = -1;
+
+#define ETH_HEADER_SIZE	14
 
 /* These are the Netware-ish names for the different Ethernet frame types.
 	EthernetII: The ethernet with a Type field instead of a length field
@@ -55,8 +64,13 @@
 void
 capture_eth(const u_char *pd, guint32 cap_len, packet_counts *ld) {
   guint16 etype;
-  int        offset = 14;
-  int   	ethhdr_type;	/* the type of ethernet frame */
+  int     offset = ETH_HEADER_SIZE;
+  int     ethhdr_type;	/* the type of ethernet frame */
+
+  if (cap_len < ETH_HEADER_SIZE) {
+    ld->other++;
+    return;
+  }
   
   etype = (pd[12] << 8) | pd[13];
 
@@ -93,76 +107,94 @@ capture_eth(const u_char *pd, guint32 cap_len, packet_counts *ld) {
 }
 
 void
-dissect_eth(const u_char *pd, frame_data *fd, proto_tree *tree) {
+dissect_eth(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   guint16    etype, length;
-  int        offset = 14;
   proto_tree *fh_tree = NULL;
   proto_item *ti;
-  int   	ethhdr_type;	/* the type of ethernet frame */
+  int        ethhdr_type;	/* the type of ethernet frame */
+  
+  if (fd->cap_len < ETH_HEADER_SIZE) {
+    dissect_data(pd, offset, fd, tree);
+    return;
+  }
 
   if (check_col(fd, COL_RES_DL_DST))
-    col_add_str(fd, COL_RES_DL_DST, get_ether_name((u_char *)&pd[0]));
+    col_add_str(fd, COL_RES_DL_DST, get_ether_name((u_char *)&pd[offset+0]));
   if (check_col(fd, COL_RES_DL_SRC))
-    col_add_str(fd, COL_RES_DL_SRC, get_ether_name((u_char *)&pd[6]));
+    col_add_str(fd, COL_RES_DL_SRC, get_ether_name((u_char *)&pd[offset+6]));
   if (check_col(fd, COL_UNRES_DL_DST))
-    col_add_str(fd, COL_UNRES_DL_DST, ether_to_str((u_char *)&pd[0]));
+    col_add_str(fd, COL_UNRES_DL_DST, ether_to_str((u_char *)&pd[offset+0]));
   if (check_col(fd, COL_UNRES_DL_SRC))
-    col_add_str(fd, COL_UNRES_DL_SRC, ether_to_str((u_char *)&pd[6]));
+    col_add_str(fd, COL_UNRES_DL_SRC, ether_to_str((u_char *)&pd[offset+6]));
   if (check_col(fd, COL_PROTOCOL))
-    col_add_str(fd, COL_PROTOCOL, "N/A");
-  if (check_col(fd, COL_INFO))
-    col_add_str(fd, COL_INFO, "Ethernet II");
+    col_add_str(fd, COL_PROTOCOL, "Ethernet");
 
-  etype = (pd[12] << 8) | pd[13];
+  etype = (pd[offset+12] << 8) | pd[offset+13];
 
 	/* either ethernet802.3 or ethernet802.2 */
   if (etype <= IEEE_802_3_MAX_LEN) {
     length = etype;
 
-  /* Is there an 802.2 layer? I can tell by looking at the first 2
-     bytes after the 802.3 header. If they are 0xffff, then what
-     follows the 802.3 header is an IPX payload, meaning no 802.2.
-     (IPX/SPX is they only thing that can be contained inside a
-     straight 802.3 packet). A non-0xffff value means that there's an
-     802.2 layer inside the 802.3 layer */
-    if (pd[14] == 0xff && pd[15] == 0xff) {
+    /* Is there an 802.2 layer? I can tell by looking at the first 2
+       bytes after the 802.3 header. If they are 0xffff, then what
+       follows the 802.3 header is an IPX payload, meaning no 802.2.
+       (IPX/SPX is they only thing that can be contained inside a
+       straight 802.3 packet). A non-0xffff value means that there's an
+       802.2 layer inside the 802.3 layer */
+    if (pd[offset+14] == 0xff && pd[offset+15] == 0xff) {
       ethhdr_type = ETHERNET_802_3;
     }
     else {
       ethhdr_type = ETHERNET_802_2;
     }
 
-    if (check_col(fd, COL_INFO))
-      col_add_str(fd, COL_INFO, "802.3");
+    if (check_col(fd, COL_INFO)) {
+      col_add_fstr(fd, COL_INFO, "IEEE 802.3 %s",
+		(ethhdr_type == ETHERNET_802_3 ? "Raw " : ""));
+    }
     if (tree) {
-      ti = proto_tree_add_item(tree, 0, offset,
-        "IEEE 802.3 %s", (ethhdr_type == ETHERNET_802_3 ? "Raw " : ""));
 
-      fh_tree = proto_tree_new();
-      proto_item_add_subtree(ti, fh_tree, ETT_IEEE8023);
-      proto_tree_add_item(fh_tree, 0, 6, "Destination: %s (%s)",
-	ether_to_str((guint8 *) &pd[0]),
-        get_ether_name((u_char *) &pd[0]));
-      proto_tree_add_item(fh_tree, 6, 6, "Source: %s (%s)",
-	ether_to_str((guint8 *) &pd[6]),
-	get_ether_name((u_char *)&pd[6]));
-      proto_tree_add_item(fh_tree, 12, 2, "Length: %d", length);
+	ti = proto_tree_add_item_format(tree, proto_eth, 0, ETH_HEADER_SIZE,
+		NULL, "IEEE 802.3 %s", (ethhdr_type == ETHERNET_802_3 ? "Raw " : ""));
+
+	fh_tree = proto_item_add_subtree(ti, ETT_IEEE8023);
+
+	proto_tree_add_item(fh_tree, hf_eth_dst, 0, 6, &pd[offset+0]);
+	proto_tree_add_item(fh_tree, hf_eth_src, 6, 6, &pd[offset+6]);
+	proto_tree_add_item(fh_tree, hf_eth_len, 12, 2, length);
+
+	/* Convert the LLC length from the 802.3 header to a total
+	   length, by adding in the Ethernet header size, and set
+	   the payload and captured-payload lengths to the minima
+	   of the total length and the frame lengths. */
+	length += ETH_HEADER_SIZE;
+	if (pi.len > length)
+	  pi.len = length;
+	if (pi.captured_len > length)
+	  pi.captured_len = length;
     }
 
   } else {
     ethhdr_type = ETHERNET_II;
+    if (check_col(fd, COL_INFO))
+      col_add_str(fd, COL_INFO, "Ethernet II");
     if (tree) {
-      ti = proto_tree_add_item(tree, 0, 14, "Ethernet II");
-      fh_tree = proto_tree_new();
-      proto_item_add_subtree(ti, fh_tree, ETT_ETHER2);
-      proto_tree_add_item(fh_tree, 0, 6, "Destination: %s (%s)",
-	ether_to_str((guint8 *) &pd[0]),
-	get_ether_name((u_char *)&pd[0]));
-      proto_tree_add_item(fh_tree, 6, 6, "Source: %s (%s)",
-        ether_to_str((guint8 *) &pd[6]),
-        get_ether_name((u_char *)&pd[6]));
+
+	ti = proto_tree_add_item_format(tree, proto_eth, 0, ETH_HEADER_SIZE,
+		NULL, "Ethernet II");
+
+	fh_tree = proto_item_add_subtree(ti, ETT_ETHER2);
+
+	proto_tree_add_item_format(fh_tree, hf_eth_dst, 0, 6, &pd[offset+0],
+		"Destination: %s (%s)", ether_to_str((guint8 *) &pd[offset+0]),
+		get_ether_name((u_char *) &pd[offset+0]));
+
+	proto_tree_add_item_format(fh_tree, hf_eth_src, 6, 6, &pd[offset+6],
+		"Source: %s (%s)", ether_to_str((guint8 *) &pd[offset+6]),
+		get_ether_name((u_char *) &pd[offset+6]));
     }
   }
+  offset += ETH_HEADER_SIZE;
 
   switch (ethhdr_type) {
     case ETHERNET_802_3:
@@ -172,8 +204,30 @@ dissect_eth(const u_char *pd, frame_data *fd, proto_tree *tree) {
       dissect_llc(pd, offset, fd, tree);
       break;
     case ETHERNET_II:
-      ethertype(etype, offset, pd, fd, tree, fh_tree);
+      ethertype(etype, offset, pd, fd, tree, fh_tree, hf_eth_type);
       break;
   }
 }
 
+void
+proto_register_eth(void)
+{
+	static hf_register_info hf[] = {
+
+		{ &hf_eth_dst,
+		{ "Destination",	"eth.dst", FT_ETHER, NULL }},
+
+		{ &hf_eth_src,
+		{ "Source",		"eth.src", FT_ETHER, NULL }},
+
+		{ &hf_eth_len,
+		{ "Length",		"eth.len", FT_UINT16, NULL }},
+
+		/* registered here but handled in ethertype.c */
+		{ &hf_eth_type,
+		{ "Type",		"eth.type", FT_VALS_UINT16, VALS(etype_vals) }}
+	};
+
+	proto_eth = proto_register_protocol ("Ethernet", "eth" );
+	proto_register_field_array(proto_eth, hf, array_length(hf));
+}
