@@ -3,7 +3,7 @@
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
- * $Id: packet-giop.c,v 1.2 1999/03/23 03:14:37 gram Exp $
+ * $Id: packet-giop.c,v 1.6 1999/09/17 05:56:53 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -40,6 +40,8 @@
 #include <glib.h>
 #include "packet.h"
 
+static int proto_giop = -1;
+
 /*
  * GIOP / IIOP types definition - OMG CORBA 2.x / GIOP 1.[01]
  * See OMG WEB site <http://www.omg.org> - CORBA+IIOP 2.2 (98-02-01.ps)
@@ -209,7 +211,7 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 
 #define END_OF_GIOP_MESSAGE (offset - first_offset - GIOP_HEADER_SIZE)
 
-  if (fd->cap_len < offset + GIOP_HEADER_SIZE) {
+  if (!BYTES_ARE_IN_FRAME(offset, GIOP_HEADER_SIZE)) {
     dissect_data(pd, offset, fd, tree);
     return;
   }
@@ -258,27 +260,25 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
   }
 
   if (tree) {
-    ti = proto_tree_add_item(tree, offset, 
-			  GIOP_HEADER_SIZE + message_size,
-			  "General Inter-ORB Protocol");
-    clnp_tree = proto_tree_new();
-    proto_item_add_subtree(ti, clnp_tree, ETT_GIOP);
-    proto_tree_add_item(clnp_tree, offset,      4,
+    ti = proto_tree_add_item(tree, proto_giop, offset, 
+			  GIOP_HEADER_SIZE + message_size, NULL);
+    clnp_tree = proto_item_add_subtree(ti, ETT_GIOP);
+    proto_tree_add_text(clnp_tree, offset,      4,
 		     "Magic number: %s", GIOP_MAGIC);
-    proto_tree_add_item(clnp_tree, offset +  4, 2, 
+    proto_tree_add_text(clnp_tree, offset +  4, 2, 
 		     "Version: %d.%d", 
 		     header.GIOP_version.major,
 		     header.GIOP_version.minor);
     switch(minor_version) {
       case 1  :
-	proto_tree_add_item(clnp_tree, offset +  6, 1, 
+	proto_tree_add_text(clnp_tree, offset +  6, 1, 
 			 "Flags: 0x%02x (%s%s)", 
 			 header.flags,
 			 (big_endian) ? "little" : "big",
 			 (header.flags & 0x02) ? " fragment" : "");
 	break;
       case 0  :
-	proto_tree_add_item(clnp_tree, offset +  6, 1, 
+	proto_tree_add_text(clnp_tree, offset +  6, 1, 
 			 "Byte ordering: %s endian",
 			 (big_endian) ? "little" : "big");
 	break;
@@ -286,7 +286,7 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	break;
     } /* minor_version */
 
-    proto_tree_add_item(clnp_tree, offset +  7, 1, 
+    proto_tree_add_text(clnp_tree, offset +  7, 1, 
 		     "Message type: %s",
 		     (header.message_type == Request) ? "Request" :
 		     (header.message_type == Reply) ? "Reply" :
@@ -297,14 +297,14 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 		     (header.message_type == MessageError) ? "MessageError" :
 		     (header.message_type == Fragment) ? "Fragment" : "?");
 
-    proto_tree_add_item(clnp_tree, offset +  8, 4, 
+    proto_tree_add_text(clnp_tree, offset +  8, 4, 
 		     "Message size: %d", message_size);
 
   } /* tree */
 
   offset += GIOP_HEADER_SIZE;
 
-  if (fd->cap_len < offset + message_size) {
+  if (!BYTES_ARE_IN_FRAME(offset, message_size)) {
     dissect_data(pd, offset, fd, tree);
     return;
   }
@@ -332,12 +332,12 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	}
 
 	if (tree) {
-	  proto_tree_add_item(clnp_tree, offset, sizeof(context_id),
+	  proto_tree_add_text(clnp_tree, offset, sizeof(context_id),
 			   "Context id: %d", context_id);
-	  proto_tree_add_item(clnp_tree, offset + sizeof(context_id),
+	  proto_tree_add_text(clnp_tree, offset + sizeof(context_id),
 			   sizeof(sequence_length),
 			   "Sequence length: %d", sequence_length);
-	  proto_tree_add_item(clnp_tree,
+	  proto_tree_add_text(clnp_tree,
 			   offset + 
 			   sizeof(context_id) + sizeof(sequence_length),
 			   sequence_length,
@@ -367,13 +367,13 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	  request_id = (big_endian)? pntohl(&request_1_1.request_id) :
 	    pletohl(&request_1_1.request_id);
 	  if (tree) {
-	    proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	    proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			     "Request id: %d", request_id);
-	    proto_tree_add_item(clnp_tree, offset + sizeof(request_id),
+	    proto_tree_add_text(clnp_tree, offset + sizeof(request_id),
 			     sizeof(request_1_1.response_expected),
 			     "Response expected: %d", 
 			     response_expected);
-	    proto_tree_add_item(clnp_tree, offset + sizeof(request_id) +
+	    proto_tree_add_text(clnp_tree, offset + sizeof(request_id) +
 			     sizeof(request_1_1.response_expected),
 			     3,
 			     "Reserved");
@@ -387,9 +387,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	  request_id = (big_endian)? pntohl(&request_1_0.request_id) :
 	    pletohl(&request_1_0.request_id);
 	  if (tree) {
-	    proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	    proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			     "Request id: %d", request_id);
-	    proto_tree_add_item(clnp_tree, offset + sizeof(request_id),
+	    proto_tree_add_text(clnp_tree, offset + sizeof(request_id),
 			     sizeof(request_1_0.response_expected),
 			     "Response expected: %d", 
 			     response_expected);
@@ -412,9 +412,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(sequence_length),
+	proto_tree_add_text(clnp_tree, offset, sizeof(sequence_length),
 			 "Object key length: %d", sequence_length);
-	proto_tree_add_item(clnp_tree, offset + sizeof(sequence_length),
+	proto_tree_add_text(clnp_tree, offset + sizeof(sequence_length),
 			 sequence_length,
 			 "Object key: %s",
 			 print_object_key(sequence_length, 
@@ -435,13 +435,13 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
       }
        
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(sequence_length),
+	proto_tree_add_text(clnp_tree, offset, sizeof(sequence_length),
 			 "Operation length: %d", sequence_length);
-	proto_tree_add_item(clnp_tree, offset + sizeof(sequence_length), 
+	proto_tree_add_text(clnp_tree, offset + sizeof(sequence_length), 
 			 sequence_length,
 			 "Operation: %s",
 			 &pd[offset+sizeof(sequence_length)]);
-	proto_tree_add_item(clnp_tree, offset +
+	proto_tree_add_text(clnp_tree, offset +
 			 sizeof(sequence_length)+ sequence_length,
 			 message_size - END_OF_GIOP_MESSAGE - 
 			 sizeof(sequence_length) - sequence_length,
@@ -466,9 +466,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	pntohl(&reply.reply_status) : pletohl(&reply.reply_status);
 
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			 "Request id: %d", request_id);
-	proto_tree_add_item(clnp_tree, offset + sizeof(request_id), 
+	proto_tree_add_text(clnp_tree, offset + sizeof(request_id), 
 			 sizeof(reply_status),
 			 "Reply status: %s",
 			 reply_status == NO_EXCEPTION ? "no exception" :
@@ -504,9 +504,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	}
 
 	if (tree) {
-	  proto_tree_add_item(clnp_tree, offset, sizeof(sequence_length),
+	  proto_tree_add_text(clnp_tree, offset, sizeof(sequence_length),
 			   "Exception length: %d", sequence_length);
-	  proto_tree_add_item(clnp_tree, offset + sizeof(sequence_length), 
+	  proto_tree_add_text(clnp_tree, offset + sizeof(sequence_length), 
 			   sequence_length,
 			   "Exception id: %s",
 			   &pd[offset+sizeof(sequence_length)]);
@@ -522,9 +522,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	  pletohl(&pd[offset+sizeof(minor_code_value)]);
 	
 	if (tree) {
-	  proto_tree_add_item(clnp_tree, offset, sizeof(minor_code_value),
+	  proto_tree_add_text(clnp_tree, offset, sizeof(minor_code_value),
 			   "Minor code value: %d", minor_code_value);
-	  proto_tree_add_item(clnp_tree, offset + sizeof(minor_code_value),
+	  proto_tree_add_text(clnp_tree, offset + sizeof(minor_code_value),
 			   sizeof(completion_status),
 			   "Completion Status: %d",
 			   completion_status);
@@ -543,9 +543,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	}
 
 	if (tree) {
-	  proto_tree_add_item(clnp_tree, offset, sizeof(sequence_length),
+	  proto_tree_add_text(clnp_tree, offset, sizeof(sequence_length),
 			   "Exception length: %d", sequence_length);
-	  proto_tree_add_item(clnp_tree, offset + sizeof(sequence_length), 
+	  proto_tree_add_text(clnp_tree, offset + sizeof(sequence_length), 
 			   sequence_length,
 			   "Exception id: %s",
 			   &pd[offset+sizeof(sequence_length)]);
@@ -563,9 +563,9 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	}
 
 	if (tree && sequence_length) {
-	  proto_tree_add_item(clnp_tree, offset, sizeof(sequence_length),
+	  proto_tree_add_text(clnp_tree, offset, sizeof(sequence_length),
 			   "Exception member length: %d", sequence_length);
-	  proto_tree_add_item(clnp_tree, offset + sizeof(sequence_length), 
+	  proto_tree_add_text(clnp_tree, offset + sizeof(sequence_length), 
 			   sequence_length,
 			   "Exception member: %s",
 			   &pd[offset+sizeof(sequence_length)]);
@@ -577,7 +577,7 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
       else {
 	
 	if (tree) {
-	  proto_tree_add_item(clnp_tree, offset,
+	  proto_tree_add_text(clnp_tree, offset,
 			   message_size - END_OF_GIOP_MESSAGE,
 			   "Reply body: <not shown>");
 	}
@@ -597,13 +597,13 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	pletohl(&pd[offset+sizeof(request_id)]);
 
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			 "Request id: %d", request_id);
-	proto_tree_add_item(clnp_tree, offset + sizeof(request_id), 
+	proto_tree_add_text(clnp_tree, offset + sizeof(request_id), 
 			 sizeof(sequence_length),
 			 "Object key length: %d", sequence_length);
 	offset += sizeof(request_id) + sizeof(sequence_length);
-	proto_tree_add_item(clnp_tree,
+	proto_tree_add_text(clnp_tree,
 			 offset,
 			 sequence_length,
 			 "Object key: %s", 
@@ -626,14 +626,14 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	pntohl(&locate_rep.locate_status) : pletohl(&locate_rep.locate_status);
 
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			 "Request id: %d", request_id);
-	proto_tree_add_item(clnp_tree, offset + sizeof(request_id), 
+	proto_tree_add_text(clnp_tree, offset + sizeof(request_id), 
 			 sizeof(locate_status),
 			 "Locate status: %d", locate_status);
 	offset += sizeof(request_id) + sizeof(locate_status);
 	if (locate_status == OBJECT_FORWARD) {
-	  proto_tree_add_item(clnp_tree, offset,
+	  proto_tree_add_text(clnp_tree, offset,
 			   message_size - END_OF_GIOP_MESSAGE,
 			   "Locate reply body: <not shown>");
 	}
@@ -655,7 +655,7 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
       if (tree) {
-	proto_tree_add_item(clnp_tree, offset, sizeof(request_id),
+	proto_tree_add_text(clnp_tree, offset, sizeof(request_id),
 			 "Request id: %d", request_id);
       }
 
@@ -691,9 +691,20 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 
   offset = first_offset + GIOP_HEADER_SIZE + message_size;
 
-  if (offset < fd->cap_len) {
+  if (IS_DATA_IN_FRAME(offset)) {
     dissect_data(pd, offset, fd, tree);
   }
 
 } /* dissect_giop */
 
+void
+proto_register_giop(void)
+{
+/*        static hf_register_info hf[] = {
+                { &variable,
+                { "Name",           "giop.abbreviation", TYPE, VALS_POINTER }},
+        };*/
+
+        proto_giop = proto_register_protocol("General Inter-ORB Protocol", "giop");
+ /*       proto_register_field_array(proto_giop, hf, array_length(hf));*/
+}
