@@ -3,7 +3,7 @@
  *
  * Guy Harris <guy@netapp.com>
  *
- * $Id: packet-http.c,v 1.3 1999/03/30 04:41:01 guy Exp $
+ * $Id: packet-http.c,v 1.9 1999/09/17 05:56:54 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -27,7 +27,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -39,10 +41,13 @@
 #include <glib.h>
 #include "packet.h"
 
+static int proto_http = -1;
+
 static int is_http_request_or_reply(const u_char *data, int linelen);
 
 void dissect_http(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
+	gboolean	is_ipp = (pi.srcport == 631 || pi.destport == 631);
 	proto_tree	*http_tree;
 	proto_item	*ti;
 	const u_char	*data, *dataend;
@@ -54,7 +59,7 @@ void dissect_http(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	dataend = data + END_OF_FRAME;
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "HTTP");
+		col_add_str(fd, COL_PROTOCOL, is_ipp ? "IPP" : "HTTP");
 	if (check_col(fd, COL_INFO)) {
 		/*
 		 * Put the first line from the buffer into the summary,
@@ -70,10 +75,8 @@ void dissect_http(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	}
 
 	if (tree) {
-		ti = proto_tree_add_item(tree, offset, END_OF_FRAME,
-		  "Hypertext Transfer Protocol");
-		http_tree = proto_tree_new();
-		proto_item_add_subtree(ti, http_tree, ETT_HTTP);
+		ti = proto_tree_add_item(tree, proto_http, offset, END_OF_FRAME, NULL);
+		http_tree = proto_item_add_subtree(ti, ETT_HTTP);
 
 		while (data < dataend) {
 			/*
@@ -160,15 +163,17 @@ void dissect_http(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 			/*
 			 * Put this line.
 			 */
-			proto_tree_add_item(http_tree, offset, linelen, "%s",
+			proto_tree_add_text(http_tree, offset, linelen, "%s",
 			    format_text(data, linelen));
 			offset += linelen;
 			data = lineend;
 		}
 
 		if (data < dataend) {
-			proto_tree_add_item(http_tree, offset, END_OF_FRAME,
-			    "Data (%d bytes)", END_OF_FRAME);
+			if (is_ipp)
+				dissect_ipp(pd, offset, fd, tree);
+			else
+				dissect_data(&pd[offset], offset, fd, http_tree);
 		}
 	}
 }
@@ -206,3 +211,15 @@ is_http_request_or_reply(const u_char *data, int linelen)
 	}
 	return FALSE;
 }
+
+void
+proto_register_http(void)
+{
+/*        static hf_register_info hf[] = {
+                { &variable,
+                { "Name",           "http.abbreviation", TYPE, VALS_POINTER }},
+        };*/
+
+        proto_http = proto_register_protocol("Hypertext Transfer Protocol", "http");
+ /*       proto_register_field_array(proto_http, hf, array_length(hf));*/
+}
