@@ -2,7 +2,7 @@
  * Routines for NetWare's IPX
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
- * $Id: packet-ipx.c,v 1.18 1999/03/23 03:14:39 gram Exp $
+ * $Id: packet-ipx.c,v 1.27 1999/09/15 22:33:17 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -48,6 +48,22 @@
   And from the ncpfs source code by Volker Lendecke
 
 */
+	
+static int proto_ipx = -1;
+static int hf_ipx_checksum = -1;
+static int hf_ipx_len = -1;
+static int hf_ipx_hops = -1;
+static int hf_ipx_packet_type = -1;
+static int hf_ipx_dnet = -1;
+static int hf_ipx_dnode = -1;
+static int hf_ipx_dsocket = -1;
+static int hf_ipx_snet = -1;
+static int hf_ipx_snode = -1;
+static int hf_ipx_ssocket = -1;
+
+static int proto_spx = -1;
+static int proto_ipxrip = -1;
+static int proto_sap = -1;
 
 static void
 dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
@@ -58,9 +74,11 @@ dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
 static void
 dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
 
+typedef	void	(dissect_func_t)(const u_char *, int, frame_data *, proto_tree *);
+
 struct port_info {
 	guint16	port;
-	void	(*func) (const u_char *, int, frame_data *, proto_tree *);
+	dissect_func_t *func;
 	char	*text;
 };
 
@@ -77,18 +95,41 @@ struct server_info {
 /* ================================================================= */
 /* IPX                                                               */
 /* ================================================================= */
+
+#define IPX_SOCKET_NCP			0x0451
+#define IPX_SOCKET_SAP			0x0452
+#define IPX_SOCKET_IPXRIP		0x0453
+#define IPX_SOCKET_NETBIOS		0x0455
+#define IPX_SOCKET_DIAGNOSTIC		0x0456
+#define IPX_SOCKET_SERIALIZATION	0x0457
+#define IPX_SOCKET_NWLINK_SMB_NAMEQUERY	0x0551
+#define IPX_SOCKET_NWLINK_SMB_DGRAM	0x0553
+#define IPX_SOCKET_ATTACHMATE_GW	0x055d
+#define IPX_SOCKET_IPX_MESSAGE		0x4001
+
 static struct port_info	ports[] = {
-	{ 0x0451, dissect_ncp,		"NCP" },
-	{ 0x0452, dissect_sap,		"SAP" },
-	{ 0x0453, dissect_ipxrip, 	"RIP" },
-	{ 0x0455, NULL,				"NetBIOS" },
-	{ 0x0456, NULL,				"Diagnostic" },
-	{ 0x0457, NULL,				"Serialization" },
-	{ 0x0551, NULL,				"NWLink SMB Name Query" },
-	{ 0x0553, dissect_nwlink_dg,"NWLink SMB Datagram" },
-	{ 0x055d, NULL,				"Attachmate Gateway" },
-	{ 0x4001, NULL,				"IPX Message" },
-	{ 0x0000, NULL,				NULL }
+	{ IPX_SOCKET_NCP,			dissect_ncp,
+				"NCP" },
+	{ IPX_SOCKET_SAP,			dissect_sap,
+				"SAP" },
+	{ IPX_SOCKET_IPXRIP,			dissect_ipxrip,
+				"RIP" },
+	{ IPX_SOCKET_NETBIOS,			NULL,
+				"NetBIOS" },
+	{ IPX_SOCKET_DIAGNOSTIC,		NULL,
+				"Diagnostic" },
+	{ IPX_SOCKET_SERIALIZATION,		NULL,
+				"Serialization" },
+	{ IPX_SOCKET_NWLINK_SMB_NAMEQUERY,	NULL,
+				"NWLink SMB Name Query" },
+	{ IPX_SOCKET_NWLINK_SMB_DGRAM,		dissect_nwlink_dg,
+				"NWLink SMB Datagram" },
+	{ IPX_SOCKET_ATTACHMATE_GW,		NULL,
+				"Attachmate Gateway" },
+	{ IPX_SOCKET_IPX_MESSAGE,		NULL,
+				"IPX Message" },
+	{ 0x0000,				NULL,
+				NULL }
 };
 
 static char*
@@ -104,7 +145,7 @@ port_text(guint16 port) {
 	return "Unknown";
 }
 
-static void*
+static dissect_func_t*
 port_func(guint16 port) {
 	int i=0;
 
@@ -117,28 +158,36 @@ port_func(guint16 port) {
 	return NULL;
 }
 
-char *
-ipx_packet_type(u_char val)
-{
-	if (val == 0) {
-		return "IPX";
-	}
-	else if (val == 5) {
-		return "SPX";
-	}
-	else if (val == 17) {
-		return "NCP";
-	}
-	else if (val == 20) {
-		return "NetBIOS Broadcast";
-	}
-	else if (val >= 16 && val <= 31) {
-		return "Experimental Protocol";
-	}
-	else {
-		return "Unknown";
-	}
-}
+#define IPX_PACKET_TYPE_IPX		0
+#define IPX_PACKET_TYPE_IPX_4		4
+#define IPX_PACKET_TYPE_SPX		5
+#define IPX_PACKET_TYPE_NCP		17
+#define IPX_PACKET_TYPE_WANBCAST	20
+
+static const value_string ipx_packet_type_vals[] = {
+	{ IPX_PACKET_TYPE_IPX,		"IPX" },
+	{ IPX_PACKET_TYPE_IPX_4,	"IPX" },
+				/* NetMon calls it "IPX" */
+	{ IPX_PACKET_TYPE_SPX,		"SPX" },
+	{ 16,				"Experimental Protocol" },
+	{ IPX_PACKET_TYPE_NCP,		"NCP" },
+	{ 18,				"Experimental Protocol" },
+	{ 19,				"Experimental Protocol" },
+	{ IPX_PACKET_TYPE_WANBCAST,	"NetBIOS Broadcast" },
+				/* NetMon calls it "WAN Broadcast" */
+	{ 21,				"Experimental Protocol" },
+	{ 22,				"Experimental Protocol" },
+	{ 23,				"Experimental Protocol" },
+	{ 24,				"Experimental Protocol" },
+	{ 25,				"Experimental Protocol" },
+	{ 26,				"Experimental Protocol" },
+	{ 27,				"Experimental Protocol" },
+	{ 28,				"Experimental Protocol" },
+	{ 29,				"Experimental Protocol" },
+	{ 30,				"Experimental Protocol" },
+	{ 31,				"Experimental Protocol" },
+	{ 0,				NULL }
+};
 
 gchar*
 ipxnet_to_string(const guint8 *ad)
@@ -184,22 +233,38 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	proto_item	*ti;
 	guint8		ipx_type, ipx_hops;
 	guint16		ipx_checksum, ipx_length;
+	int		len;
 	guint8		*ipx_snode, *ipx_dnode, *ipx_snet, *ipx_dnet;
 
 	gchar		*str_dnet, *str_snet;
 	guint16		ipx_dsocket, ipx_ssocket;
-	void		(*dissect) (const u_char *, int, frame_data *, proto_tree *);
+	dissect_func_t	*dissect;
+	guint32		ipx_dnet_val, ipx_snet_val;
 
 	/* Calculate here for use in pinfo and in tree */
 	ipx_dnet = (guint8*)&pd[offset+6];
 	ipx_snet = (guint8*)&pd[offset+18];
 	str_dnet = ipxnet_to_string(ipx_dnet);
 	str_snet = ipxnet_to_string(ipx_snet);
+	ipx_dnet_val = pntohl(ipx_dnet);
+	ipx_snet_val = pntohl(ipx_snet);
 	ipx_dsocket = pntohs(&pd[offset+16]);
 	ipx_ssocket = pntohs(&pd[offset+28]);
 	ipx_dnode = (guint8*)&pd[offset+10];
 	ipx_snode = (guint8*)&pd[offset+22];
 	ipx_type = pd[offset+5];
+	ipx_length = pntohs(&pd[offset+2]);
+
+	/* Length of IPX datagram plus headers above it. */
+		len = ipx_length + offset;
+
+	/* Set the payload and captured-payload lengths to the minima of
+	   (the IPX length plus the length of the headers above it) and
+	   the frame lengths. */
+	if (pi.len > len)
+		pi.len = len;
+	if (pi.captured_len > len)
+		pi.captured_len = len;
 
 	if (check_col(fd, COL_RES_DL_DST))
 		col_add_str(fd, COL_RES_DL_DST,
@@ -216,45 +281,41 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	if (tree) {
 		ipx_checksum = pntohs(&pd[offset]);
-		ipx_length = pntohs(&pd[offset+2]);
 		ipx_hops = pd[offset+4];
 
-		ti = proto_tree_add_item(tree, offset, 30,
-			"Internetwork Packet Exchange");
-		ipx_tree = proto_tree_new();
-		proto_item_add_subtree(ti, ipx_tree, ETT_IPX);
-		proto_tree_add_item(ipx_tree, offset,      2, "Checksum: 0x%04x",
-				ipx_checksum);
-		proto_tree_add_item(ipx_tree, offset+2,    2, "Length: %d bytes",
-				ipx_length);
-		proto_tree_add_item(ipx_tree, offset+4,    1, "Transport Control: %d hops",
-				ipx_hops);
-		proto_tree_add_item(ipx_tree, offset+5,    1, "Packet Type: %s",
-			ipx_packet_type(ipx_type));
-		proto_tree_add_item(ipx_tree, offset+6,    4, "Destination Network: %s",
-			str_dnet);
-		proto_tree_add_item(ipx_tree, offset+10,   6, "Destination Node: %s",
-			ether_to_str(ipx_dnode));
-		proto_tree_add_item(ipx_tree, offset+16,   2,
-			"Destination Socket: %s (0x%04X)", port_text(ipx_dsocket),
-			ipx_dsocket);
-		proto_tree_add_item(ipx_tree, offset+18,   4, "Source Network: %s",
-			str_snet);
-		proto_tree_add_item(ipx_tree, offset+22,   6, "Source Node: %s",
-			ether_to_str(ipx_snode));
-		proto_tree_add_item(ipx_tree, offset+28,   2,
-			"Source Socket: %s (0x%04X)", port_text(ipx_ssocket), ipx_ssocket);
+		ti = proto_tree_add_item(tree, proto_ipx, offset, 30, NULL);
+		ipx_tree = proto_item_add_subtree(ti, ETT_IPX);
+		proto_tree_add_item_format(ipx_tree, hf_ipx_checksum, offset, 2, ipx_checksum,
+			"Checksum: 0x%04x", ipx_checksum);
+		proto_tree_add_item_format(ipx_tree, hf_ipx_len, offset+2, 2, ipx_length,
+			"Length: %d bytes", ipx_length);
+		proto_tree_add_item_format(ipx_tree, hf_ipx_hops, offset+4, 1, ipx_hops,
+			"Transport Control: %d hops", ipx_hops);
+		proto_tree_add_item(ipx_tree, hf_ipx_packet_type, offset+5, 1, ipx_type);
+		proto_tree_add_item(ipx_tree, hf_ipx_dnet, offset+6, 4, ipx_dnet_val);
+		proto_tree_add_item(ipx_tree, hf_ipx_dnode, offset+10, 6, ipx_dnode);
+		proto_tree_add_item_format(ipx_tree, hf_ipx_dsocket, offset+16, 2,
+			ipx_dsocket, "Destination Socket: %s (0x%04X)",
+			port_text(ipx_dsocket), ipx_dsocket);
+		proto_tree_add_item(ipx_tree, hf_ipx_snet, offset+18, 4, ipx_snet_val);
+		proto_tree_add_item(ipx_tree, hf_ipx_snode, offset+22, 6, ipx_snode);
+		proto_tree_add_item_format(ipx_tree, hf_ipx_ssocket, offset+28, 2,
+			ipx_ssocket, "Source Socket: %s (0x%04X)", port_text(ipx_ssocket),
+			ipx_ssocket);
 	}
 	offset += 30;
 
 	switch (ipx_type) {
-		case 5: /* SPX */
+		case IPX_PACKET_TYPE_SPX:
 			dissect_spx(pd, offset, fd, tree);
 			break;
 
-		case 17: /* NCP */
+		case IPX_PACKET_TYPE_NCP:
+			/* Is the destination node 00:00:00:00:00:01 ? */
 			if (pntohl(ipx_dnode) == 0 && pntohs(ipx_dnode + 4) == 1)
 				nw_server_address = pntohl(ipx_dnet);
+
+			/* Is the source node 00:00:00:00:00:01 ? */
 			else if (pntohl(ipx_snode) == 0 && pntohs(ipx_snode + 4) == 1)
 				nw_server_address = pntohl(ipx_snet);
 			else
@@ -263,14 +324,17 @@ dissect_ipx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 			dissect_ncp(pd, offset, fd, tree);
 			break;
 
-		case 20: /* NetBIOS */
-			if (ipx_dsocket == 0x0455) {
-				dissect_nbipx_ns(pd, offset, fd, tree);
+		case IPX_PACKET_TYPE_WANBCAST:
+		case IPX_PACKET_TYPE_IPX_4:
+			if (ipx_dsocket == IPX_SOCKET_NETBIOS) {
+				dissect_nbipx(pd, offset, fd, tree);
 				break;
 			}
 			/* else fall through */
 
 		case 0: /* IPX, fall through to default */
+			/* XXX - should type 0's be dissected as NBIPX
+			   if they're aimed at the NetBIOS socket? */
 		default:
 			dissect = port_func(ipx_dsocket);
 			if (dissect) {
@@ -329,7 +393,6 @@ spx_datastream(u_char type)
 
 static void
 dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
-
 	proto_tree	*spx_tree;
 	proto_item	*ti;
 
@@ -339,31 +402,30 @@ dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 		col_add_str(fd, COL_INFO, "SPX");
 
 	if (tree) {
-		ti = proto_tree_add_item(tree, offset, 12, "Sequenced Packet Exchange");
-		spx_tree = proto_tree_new();
-		proto_item_add_subtree(ti, spx_tree, ETT_SPX);
+		ti = proto_tree_add_item(tree, proto_spx, offset, 12, NULL);
+		spx_tree = proto_item_add_subtree(ti, ETT_SPX);
 
-		proto_tree_add_item(spx_tree, offset,      1,
+		proto_tree_add_text(spx_tree, offset,      1,
 			"Connection Control: %s (0x%02X)",
 			spx_conn_ctrl(pd[offset]), pd[offset]);
 
-		proto_tree_add_item(spx_tree, offset+1,     1,
+		proto_tree_add_text(spx_tree, offset+1,     1,
 			"Datastream Type: %s (0x%02X)",
 			spx_datastream(pd[offset+1]), pd[offset+1]);
 
-		proto_tree_add_item(spx_tree, offset+2,     2,
+		proto_tree_add_text(spx_tree, offset+2,     2,
 			"Source Connection ID: %d", pntohs( &pd[offset+2] ) );
 
-		proto_tree_add_item(spx_tree, offset+4,     2,
+		proto_tree_add_text(spx_tree, offset+4,     2,
 			"Destination Connection ID: %d", pntohs( &pd[offset+4] ) );
 
-		proto_tree_add_item(spx_tree, offset+6,     2,
+		proto_tree_add_text(spx_tree, offset+6,     2,
 			"Sequence Number: %d", pntohs( &pd[offset+6] ) );
 
-		proto_tree_add_item(spx_tree, offset+8,     2,
+		proto_tree_add_text(spx_tree, offset+8,     2,
 			"Acknowledgment Number: %d", pntohs( &pd[offset+8] ) );
 
-		proto_tree_add_item(spx_tree, offset+10,     2,
+		proto_tree_add_text(spx_tree, offset+10,     2,
 			"Allocation Number: %d", pntohs( &pd[offset+10] ) );
 
 		offset += 12;
@@ -376,12 +438,11 @@ dissect_spx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 /* ================================================================= */
 static void
 dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
-
 	proto_tree	*rip_tree;
 	proto_item	*ti;
 	guint16		operation;
 	struct ipx_rt_def route;
-	int			cursor;
+	int		cursor;
 
 	char		*rip_type[2] = { "Request", "Response" };
 
@@ -399,17 +460,15 @@ dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	}
 
 	if (tree) {
-		ti = proto_tree_add_item(tree, offset, END_OF_FRAME,
-			"IPX Routing Information Protocol");
-		rip_tree = proto_tree_new();
-		proto_item_add_subtree(ti, rip_tree, ETT_IPXRIP);
+		ti = proto_tree_add_item(tree, proto_ipxrip, offset, END_OF_FRAME, NULL);
+		rip_tree = proto_item_add_subtree(ti, ETT_IPXRIP);
 
 		if (operation < 2) {
-			proto_tree_add_item(rip_tree, offset, 2,
+			proto_tree_add_text(rip_tree, offset, 2,
 			"RIP packet type: %s", rip_type[operation]);
 		}
 		else {
-			proto_tree_add_item(rip_tree, offset, 2, "Unknown RIP packet type");
+			proto_tree_add_text(rip_tree, offset, 2, "Unknown RIP packet type");
 		}
 
 		for (cursor = offset + 2; cursor < fd->cap_len; cursor += 8) {
@@ -418,14 +477,14 @@ dissect_ipxrip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 			route.ticks = pntohs(&pd[cursor+6]);
 
 			if (operation == IPX_RIP_REQUEST - 1) {
-				proto_tree_add_item(rip_tree, cursor,      8,
+				proto_tree_add_text(rip_tree, cursor,      8,
 					"Route Vector: %s, %d hop%s, %d tick%s",
 					ipxnet_to_string((guint8*)&route.network),
 					route.hops,  route.hops  == 1 ? "" : "s",
 					route.ticks, route.ticks == 1 ? "" : "s");
 			}
 			else {
-				proto_tree_add_item(rip_tree, cursor,      8,
+				proto_tree_add_text(rip_tree, cursor,      8,
 					"Route Vector: %s, %d hop%s, %d tick%s (%d ms)",
 					ipxnet_to_string((guint8*)&route.network),
 					route.hops,  route.hops  == 1 ? "" : "s",
@@ -498,7 +557,7 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	proto_tree	*sap_tree, *s_tree;
 	proto_item	*ti;
-	int			cursor;
+	int		cursor;
 	struct sap_query query;
 	struct sap_server_ident server;
 
@@ -511,7 +570,7 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "SAP");
 	if (check_col(fd, COL_INFO)) {
-		if (query.query_type < 4) {
+		if (query.query_type >= 1 && query.query_type <= 4) {
 			col_add_str(fd, COL_INFO, sap_type[query.query_type - 1]);
 		}
 		else {
@@ -520,16 +579,14 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	}
 
 	if (tree) {
-		ti = proto_tree_add_item(tree, offset, END_OF_FRAME,
-			"Service Advertising Protocol");
-		sap_tree = proto_tree_new();
-		proto_item_add_subtree(ti, sap_tree, ETT_IPXSAP);
+		ti = proto_tree_add_item(tree, proto_sap, offset, END_OF_FRAME, NULL);
+		sap_tree = proto_item_add_subtree(ti, ETT_IPXSAP);
 
-		if (query.query_type < 4) {
-			proto_tree_add_item(sap_tree, offset, 2, sap_type[query.query_type - 1]);
+		if (query.query_type >= 1 && query.query_type <= 4) {
+			proto_tree_add_text(sap_tree, offset, 2, sap_type[query.query_type - 1]);
 		}
 		else {
-			proto_tree_add_item(sap_tree, offset, 2,
+			proto_tree_add_text(sap_tree, offset, 2,
 					"Unknown SAP Packet Type %d", query.query_type);
 		}
 
@@ -544,28 +601,72 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 				server.server_port = pntohs(&pd[cursor+60]);
 				server.intermediate_network = pntohs(&pd[cursor+62]);
 
-				ti = proto_tree_add_item(sap_tree, cursor+2, 48,
+				ti = proto_tree_add_text(sap_tree, cursor+2, 48,
 					"Server Name: %s", server.server_name);
-				s_tree = proto_tree_new();
-				proto_item_add_subtree(ti, s_tree, ETT_IPXSAP_SERVER);
+				s_tree = proto_item_add_subtree(ti, ETT_IPXSAP_SERVER);
 
-				proto_tree_add_item(s_tree, cursor, 2, "Server Type: %s (0x%04X)",
+				proto_tree_add_text(s_tree, cursor, 2, "Server Type: %s (0x%04X)",
 						server_type(server.server_type), server.server_type);
-				proto_tree_add_item(s_tree, cursor+50, 4, "Network: %s",
+				proto_tree_add_text(s_tree, cursor+50, 4, "Network: %s",
 						ipxnet_to_string((guint8*)&pd[cursor+50]));
-				proto_tree_add_item(s_tree, cursor+54, 6, "Node: %s",
+				proto_tree_add_text(s_tree, cursor+54, 6, "Node: %s",
 						ether_to_str((guint8*)&pd[cursor+54]));
-				proto_tree_add_item(s_tree, cursor+60, 2, "Socket: %s (0x%04X)",
+				proto_tree_add_text(s_tree, cursor+60, 2, "Socket: %s (0x%04X)",
 						port_text(server.server_port), server.server_port);
-				proto_tree_add_item(s_tree, cursor+62, 2,
+				proto_tree_add_text(s_tree, cursor+62, 2,
 						"Intermediate Networks: %d",
 						server.intermediate_network);
 			}
 		}
 		else {  /* queries */
-			proto_tree_add_item(sap_tree, offset+2, 2, "Server Type: %s (0x%04X)",
+			proto_tree_add_text(sap_tree, offset+2, 2, "Server Type: %s (0x%04X)",
 					server_type(query.server_type), query.server_type);
 		}
 	}
 }
 
+
+void
+proto_register_ipx(void)
+{
+	static hf_register_info hf_ipx[] = {
+		{ &hf_ipx_checksum,
+		{ "Checksum",		"ipx.checksum", FT_UINT16, NULL }},
+
+		{ &hf_ipx_len,
+		{ "Length",		"ipx.len", FT_UINT16, NULL }},
+
+		{ &hf_ipx_hops,
+		{ "Transport Control (Hops)", "ipx.hops", FT_UINT8, NULL }},
+
+		{ &hf_ipx_packet_type,
+		{ "Packet Type",	"ipx.packet_type", FT_VALS_UINT8, VALS(ipx_packet_type_vals) }},
+
+		{ &hf_ipx_dnet,
+		{ "Destination Network","ipx.dstnet", FT_IPXNET, NULL }},
+
+		{ &hf_ipx_dnode,
+		{ "Destination Node",	"ipx.dstnode", FT_ETHER, NULL }},
+
+		{ &hf_ipx_dsocket,
+		{ "Destination Socket",	"ipx.dstsocket", FT_UINT16, NULL }},
+
+		{ &hf_ipx_snet,
+		{ "Source Network","ipx.srcnet", FT_IPXNET, NULL }},
+
+		{ &hf_ipx_snode,
+		{ "Source Node",	"ipx.srcnode", FT_ETHER, NULL }},
+
+		{ &hf_ipx_ssocket,
+		{ "Source Socket",	"ipx.srcsocket", FT_UINT16, NULL }},
+	};
+		
+	proto_ipx = proto_register_protocol ("Internetwork Packet eXchange", "ipx");
+	proto_register_field_array(proto_ipx, hf_ipx, array_length(hf_ipx));
+
+	proto_spx = proto_register_protocol ("Sequenced Packet eXchange", "spx");
+	
+	proto_ipxrip = proto_register_protocol ("IPX Routing Information Protocol", "ipxrip");
+
+	proto_sap = proto_register_protocol ("Service Advertisement Protocol", "sap");
+}
